@@ -19,7 +19,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
 #endif
 
 #include "php.h"
@@ -30,7 +30,6 @@
 #include "php_ini.h"
 #ifdef PHP_WIN32
 # include "windows_common.h"
-# include <win32/inet.h>
 # include <windows.h>
 # include <Ws2tcpip.h>
 # include "php_sockets.h"
@@ -52,7 +51,7 @@
 # include <sys/uio.h>
 # define set_errno(a) (errno = a)
 # include "php_sockets.h"
-# if HAVE_IF_NAMETOINDEX
+# ifdef HAVE_IF_NAMETOINDEX
 #  include <net/if.h>
 # endif
 # if defined(HAVE_LINUX_SOCK_DIAG_H)
@@ -218,15 +217,10 @@ zend_module_entry sockets_module_entry = {
 ZEND_GET_MODULE(sockets)
 #endif
 
-#ifndef HAVE_INET_NTOP
-/* inet_ntop should be used instead of inet_ntoa */
-int inet_ntoa_lock = 0;
-#endif
-
 static bool php_open_listen_sock(php_socket *sock, int port, int backlog) /* {{{ */
 {
 	struct sockaddr_in  la = {0};
-	struct hostent		*hp;
+	struct hostent	    *hp;
 
 #ifndef PHP_WIN32
 	if ((hp = php_network_gethostbyname("0.0.0.0")) == NULL) {
@@ -275,6 +269,28 @@ static bool php_accept_connect(php_socket *in_sock, php_socket *out_sock, struct
 		return 0;
 	}
 
+#if !defined(PHP_WIN32)
+	/**
+	 * accept4 could had been used but not all platforms support it (e.g. Haiku, solaris < 11.4, ...)
+	 * win32, not having any concept of child process, has no need to address it.
+	 */
+	int mode;
+
+	if ((mode = fcntl(out_sock->bsd_socket, F_GETFD)) < 0) {
+		PHP_SOCKET_ERROR(out_sock, "unable to get fcntl mode on the socket", errno);
+		return 0;
+	}
+
+	int cloexec = (mode | FD_CLOEXEC);
+
+	if (mode != cloexec) {
+		if (fcntl(out_sock->bsd_socket, F_SETFD, cloexec) < 0) {
+			PHP_SOCKET_ERROR(out_sock, "unable to set cloexec mode on the socket", errno);
+			return 0;
+		}
+	}
+#endif
+
 	out_sock->error = 0;
 	out_sock->blocking = 1;
 	out_sock->type = la->sa_family;
@@ -581,9 +597,14 @@ PHP_FUNCTION(socket_select)
 	zend_long		sec, usec = 0;
 	bool		sec_is_null = 0;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "a!a!a!l!|l", &r_array, &w_array, &e_array, &sec, &sec_is_null, &usec) == FAILURE) {
-		RETURN_THROWS();
-	}
+	ZEND_PARSE_PARAMETERS_START(4, 5)
+		Z_PARAM_ARRAY_EX2(r_array, 1, 1, 0)
+		Z_PARAM_ARRAY_EX2(w_array, 1, 1, 0)
+		Z_PARAM_ARRAY_EX2(e_array, 1, 1, 0)
+		Z_PARAM_LONG_OR_NULL(sec, sec_is_null)
+		Z_PARAM_OPTIONAL
+		Z_PARAM_LONG(usec)
+	ZEND_PARSE_PARAMETERS_END();
 
 	FD_ZERO(&rfds);
 	FD_ZERO(&wfds);
@@ -651,11 +672,13 @@ PHP_FUNCTION(socket_select)
 PHP_FUNCTION(socket_create_listen)
 {
 	php_socket	*php_sock;
-	zend_long		port, backlog = 128;
+	zend_long	port, backlog = SOMAXCONN;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "l|l", &port, &backlog) == FAILURE) {
-		RETURN_THROWS();
-	}
+	ZEND_PARSE_PARAMETERS_START(1, 2)
+		Z_PARAM_LONG(port)
+		Z_PARAM_OPTIONAL
+		Z_PARAM_LONG(backlog)
+	ZEND_PARSE_PARAMETERS_END();
 
 	object_init_ex(return_value, socket_ce);
 	php_sock = Z_SOCKET_P(return_value);
@@ -678,9 +701,9 @@ PHP_FUNCTION(socket_accept)
 	php_sockaddr_storage sa;
 	socklen_t			 php_sa_len = sizeof(sa);
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "O", &arg1, socket_ce) == FAILURE) {
-		RETURN_THROWS();
-	}
+	ZEND_PARSE_PARAMETERS_START(1, 1)
+		Z_PARAM_OBJECT_OF_CLASS(arg1, socket_ce)
+	ZEND_PARSE_PARAMETERS_END();
 
 	php_sock = Z_SOCKET_P(arg1);
 	ENSURE_SOCKET_VALID(php_sock);
@@ -701,9 +724,9 @@ PHP_FUNCTION(socket_set_nonblock)
 	zval		*arg1;
 	php_socket	*php_sock;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "O", &arg1, socket_ce) == FAILURE) {
-		RETURN_THROWS();
-	}
+	ZEND_PARSE_PARAMETERS_START(1, 1)
+		Z_PARAM_OBJECT_OF_CLASS(arg1, socket_ce)
+	ZEND_PARSE_PARAMETERS_END();
 
 	php_sock = Z_SOCKET_P(arg1);
 	ENSURE_SOCKET_VALID(php_sock);
@@ -737,9 +760,9 @@ PHP_FUNCTION(socket_set_block)
 	zval		*arg1;
 	php_socket	*php_sock;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "O", &arg1, socket_ce) == FAILURE) {
-		RETURN_THROWS();
-	}
+	ZEND_PARSE_PARAMETERS_START(1, 1)
+		Z_PARAM_OBJECT_OF_CLASS(arg1, socket_ce)
+	ZEND_PARSE_PARAMETERS_END();
 
 	php_sock = Z_SOCKET_P(arg1);
 	ENSURE_SOCKET_VALID(php_sock);
@@ -774,11 +797,13 @@ PHP_FUNCTION(socket_listen)
 {
 	zval		*arg1;
 	php_socket	*php_sock;
-	zend_long		backlog = 0;
+	zend_long	backlog = 0;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "O|l", &arg1, socket_ce, &backlog) == FAILURE) {
-		RETURN_THROWS();
-	}
+	ZEND_PARSE_PARAMETERS_START(1, 2)
+		Z_PARAM_OBJECT_OF_CLASS(arg1, socket_ce)
+		Z_PARAM_OPTIONAL
+		Z_PARAM_LONG(backlog)
+	ZEND_PARSE_PARAMETERS_END();
 
 	php_sock = Z_SOCKET_P(arg1);
 	ENSURE_SOCKET_VALID(php_sock);
@@ -797,9 +822,9 @@ PHP_FUNCTION(socket_close)
 	zval *arg1;
 	php_socket *php_socket;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "O", &arg1, socket_ce) == FAILURE) {
-		RETURN_THROWS();
-	}
+	ZEND_PARSE_PARAMETERS_START(1, 1)
+		Z_PARAM_OBJECT_OF_CLASS(arg1, socket_ce)
+	ZEND_PARSE_PARAMETERS_END();
 
 	php_socket = Z_SOCKET_P(arg1);
 	ENSURE_SOCKET_VALID(php_socket);
@@ -836,9 +861,12 @@ PHP_FUNCTION(socket_write)
 	bool   length_is_null = 1;
 	char		*str;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "Os|l!", &arg1, socket_ce, &str, &str_len, &length, &length_is_null) == FAILURE) {
-		RETURN_THROWS();
-	}
+	ZEND_PARSE_PARAMETERS_START(2, 3)
+		Z_PARAM_OBJECT_OF_CLASS(arg1, socket_ce)
+		Z_PARAM_STRING(str, str_len)
+		Z_PARAM_OPTIONAL
+		Z_PARAM_LONG_OR_NULL(length, length_is_null)
+	ZEND_PARSE_PARAMETERS_END();
 
 	php_sock = Z_SOCKET_P(arg1);
 	ENSURE_SOCKET_VALID(php_sock);
@@ -876,9 +904,12 @@ PHP_FUNCTION(socket_read)
 	int			retval;
 	zend_long		length, type = PHP_BINARY_READ;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "Ol|l", &arg1, socket_ce, &length, &type) == FAILURE) {
-		RETURN_THROWS();
-	}
+	ZEND_PARSE_PARAMETERS_START(2, 3)
+		Z_PARAM_OBJECT_OF_CLASS(arg1, socket_ce)
+		Z_PARAM_LONG(length)
+		Z_PARAM_OPTIONAL
+		Z_PARAM_LONG(type)
+	ZEND_PARSE_PARAMETERS_END();
 
 	php_sock = Z_SOCKET_P(arg1);
 	ENSURE_SOCKET_VALID(php_sock);
@@ -929,19 +960,20 @@ PHP_FUNCTION(socket_getsockname)
 	php_socket				*php_sock;
 	struct sockaddr			*sa;
 	struct sockaddr_in		*sin;
-#if HAVE_IPV6
+#ifdef HAVE_IPV6
 	struct sockaddr_in6		*sin6;
 #endif
-#ifdef HAVE_INET_NTOP
 	char					addrbuf[INET6_ADDRSTRLEN];
-#endif
 	struct sockaddr_un		*s_un;
 	const char				*addr_string;
 	socklen_t				salen = sizeof(php_sockaddr_storage);
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "Oz|z", &arg1, socket_ce, &addr, &port) == FAILURE) {
-		RETURN_THROWS();
-	}
+	ZEND_PARSE_PARAMETERS_START(2, 3)
+		Z_PARAM_OBJECT_OF_CLASS(arg1, socket_ce)
+		Z_PARAM_ZVAL(addr)
+		Z_PARAM_OPTIONAL
+		Z_PARAM_ZVAL(port)
+	ZEND_PARSE_PARAMETERS_END();
 
 	php_sock = Z_SOCKET_P(arg1);
 	ENSURE_SOCKET_VALID(php_sock);
@@ -954,7 +986,7 @@ PHP_FUNCTION(socket_getsockname)
 	}
 
 	switch (sa->sa_family) {
-#if HAVE_IPV6
+#ifdef HAVE_IPV6
 		case AF_INET6:
 			sin6 = (struct sockaddr_in6 *) sa;
 			inet_ntop(AF_INET6, &sin6->sin6_addr,  addrbuf, sizeof(addrbuf));
@@ -968,14 +1000,7 @@ PHP_FUNCTION(socket_getsockname)
 #endif
 		case AF_INET:
 			sin = (struct sockaddr_in *) sa;
-#ifdef HAVE_INET_NTOP
 			addr_string = inet_ntop(AF_INET, &sin->sin_addr, addrbuf, sizeof(addrbuf));
-#else
-			while (inet_ntoa_lock == 1);
-			inet_ntoa_lock = 1;
-			addr_string = inet_ntoa(sin->sin_addr);
-			inet_ntoa_lock = 0;
-#endif
 			ZEND_TRY_ASSIGN_REF_STRING(addr, addr_string);
 
 			if (port != NULL) {
@@ -1006,19 +1031,20 @@ PHP_FUNCTION(socket_getpeername)
 	php_socket				*php_sock;
 	struct sockaddr			*sa;
 	struct sockaddr_in		*sin;
-#if HAVE_IPV6
+#ifdef HAVE_IPV6
 	struct sockaddr_in6		*sin6;
 #endif
-#ifdef HAVE_INET_NTOP
 	char					addrbuf[INET6_ADDRSTRLEN];
-#endif
 	struct sockaddr_un		*s_un;
 	const char				*addr_string;
 	socklen_t				salen = sizeof(php_sockaddr_storage);
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "Oz|z", &arg1, socket_ce, &arg2, &arg3) == FAILURE) {
-		RETURN_THROWS();
-	}
+	ZEND_PARSE_PARAMETERS_START(2, 3)
+		Z_PARAM_OBJECT_OF_CLASS(arg1, socket_ce)
+		Z_PARAM_ZVAL(arg2)
+		Z_PARAM_OPTIONAL
+		Z_PARAM_ZVAL(arg3)
+	ZEND_PARSE_PARAMETERS_END();
 
 	php_sock = Z_SOCKET_P(arg1);
 	ENSURE_SOCKET_VALID(php_sock);
@@ -1031,7 +1057,7 @@ PHP_FUNCTION(socket_getpeername)
 	}
 
 	switch (sa->sa_family) {
-#if HAVE_IPV6
+#ifdef HAVE_IPV6
 		case AF_INET6:
 			sin6 = (struct sockaddr_in6 *) sa;
 			inet_ntop(AF_INET6, &sin6->sin6_addr, addrbuf, sizeof(addrbuf));
@@ -1047,14 +1073,7 @@ PHP_FUNCTION(socket_getpeername)
 #endif
 		case AF_INET:
 			sin = (struct sockaddr_in *) sa;
-#ifdef HAVE_INET_NTOP
 			addr_string = inet_ntop(AF_INET, &sin->sin_addr, addrbuf, sizeof(addrbuf));
-#else
-			while (inet_ntoa_lock == 1);
-			inet_ntoa_lock = 1;
-			addr_string = inet_ntoa(sin->sin_addr);
-			inet_ntoa_lock = 0;
-#endif
 			ZEND_TRY_ASSIGN_REF_STRING(arg2, addr_string);
 
 			if (arg3 != NULL) {
@@ -1081,15 +1100,17 @@ PHP_FUNCTION(socket_getpeername)
 /* {{{ Creates an endpoint for communication in the domain specified by domain, of type specified by type */
 PHP_FUNCTION(socket_create)
 {
-	zend_long	domain, type, protocol;
+	zend_long	domain, type, checktype, protocol;
 	php_socket	*php_sock;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "lll", &domain, &type, &protocol) == FAILURE) {
-		RETURN_THROWS();
-	}
+	ZEND_PARSE_PARAMETERS_START(3, 3)
+		Z_PARAM_LONG(domain)
+		Z_PARAM_LONG(type)
+		Z_PARAM_LONG(protocol)
+	ZEND_PARSE_PARAMETERS_END();
 
 	if (domain != AF_UNIX
-#if HAVE_IPV6
+#ifdef HAVE_IPV6
 		&& domain != AF_INET6
 #endif
 		&& domain != AF_INET) {
@@ -1097,9 +1118,18 @@ PHP_FUNCTION(socket_create)
 		RETURN_THROWS();
 	}
 
-	if (type > 10) {
+	checktype = type;
+#ifdef SOCK_NONBLOCK
+	checktype &= ~(SOCK_CLOEXEC | SOCK_NONBLOCK);
+#endif
+
+	if (checktype > 10) {
 		zend_argument_value_error(2, "must be one of SOCK_STREAM, SOCK_DGRAM, SOCK_SEQPACKET,"
-			" SOCK_RAW, or SOCK_RDM");
+			" SOCK_RAW, or SOCK_RDM"
+#ifdef SOCK_NONBLOCK
+			" optionally OR'ed with SOCK_CLOEXEC, SOCK_NONBLOCK"
+#endif
+		);
 		RETURN_THROWS();
 	}
 
@@ -1132,15 +1162,18 @@ PHP_FUNCTION(socket_connect)
 	zend_long				port;
 	bool				port_is_null = 1;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "Os|l!", &resource_socket, socket_ce, &addr, &addr_len, &port, &port_is_null) == FAILURE) {
-		RETURN_THROWS();
-	}
+	ZEND_PARSE_PARAMETERS_START(2, 3)
+		Z_PARAM_OBJECT_OF_CLASS(resource_socket, socket_ce)
+		Z_PARAM_STRING(addr, addr_len)
+		Z_PARAM_OPTIONAL
+		Z_PARAM_LONG_OR_NULL(port, port_is_null)
+	ZEND_PARSE_PARAMETERS_END();
 
 	php_sock = Z_SOCKET_P(resource_socket);
 	ENSURE_SOCKET_VALID(php_sock);
 
 	switch(php_sock->type) {
-#if HAVE_IPV6
+#ifdef HAVE_IPV6
 		case AF_INET6: {
 			struct sockaddr_in6 sin6 = {0};
 
@@ -1215,9 +1248,9 @@ PHP_FUNCTION(socket_strerror)
 {
 	zend_long	arg1;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "l", &arg1) == FAILURE) {
-		RETURN_THROWS();
-	}
+	ZEND_PARSE_PARAMETERS_START(1, 1)
+		Z_PARAM_LONG(arg1)
+	ZEND_PARSE_PARAMETERS_END();
 
 	if (ZEND_LONG_EXCEEDS_INT(arg1)) {
 		zend_argument_value_error(1, "must be between %d and %d", INT_MIN, INT_MAX);
@@ -1240,9 +1273,12 @@ PHP_FUNCTION(socket_bind)
 	zend_long					port = 0;
 	zend_long					retval = 0;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "Os|l", &arg1, socket_ce, &addr, &addr_len, &port) == FAILURE) {
-		RETURN_THROWS();
-	}
+	ZEND_PARSE_PARAMETERS_START(2, 3)
+		Z_PARAM_OBJECT_OF_CLASS(arg1, socket_ce)
+		Z_PARAM_STRING(addr, addr_len)
+		Z_PARAM_OPTIONAL
+		Z_PARAM_LONG(port)
+	ZEND_PARSE_PARAMETERS_END();
 
 	php_sock = Z_SOCKET_P(arg1);
 	ENSURE_SOCKET_VALID(php_sock);
@@ -1279,7 +1315,7 @@ PHP_FUNCTION(socket_bind)
 				retval = bind(php_sock->bsd_socket, (struct sockaddr *)sa, sizeof(struct sockaddr_in));
 				break;
 			}
-#if HAVE_IPV6
+#ifdef HAVE_IPV6
 		case AF_INET6:
 			{
 				struct sockaddr_in6 *sa = (struct sockaddr_in6 *) sock_type;
@@ -1318,9 +1354,12 @@ PHP_FUNCTION(socket_recv)
 	int			retval;
 	zend_long		len, flags;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "Ozll", &php_sock_res, socket_ce, &buf, &len, &flags) == FAILURE) {
-		RETURN_THROWS();
-	}
+	ZEND_PARSE_PARAMETERS_START(4, 4)
+		Z_PARAM_OBJECT_OF_CLASS(php_sock_res, socket_ce)
+		Z_PARAM_ZVAL(buf)
+		Z_PARAM_LONG(len)
+		Z_PARAM_LONG(flags)
+	ZEND_PARSE_PARAMETERS_END();
 
 	php_sock = Z_SOCKET_P(php_sock_res);
 	ENSURE_SOCKET_VALID(php_sock);
@@ -1359,9 +1398,12 @@ PHP_FUNCTION(socket_send)
 	zend_long		len, flags;
 	char		*buf;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "Osll", &arg1, socket_ce, &buf, &buf_len, &len, &flags) == FAILURE) {
-		RETURN_THROWS();
-	}
+	ZEND_PARSE_PARAMETERS_START(4, 4)
+		Z_PARAM_OBJECT_OF_CLASS(arg1, socket_ce)
+		Z_PARAM_STRING(buf, buf_len)
+		Z_PARAM_LONG(len)
+		Z_PARAM_LONG(flags)
+	ZEND_PARSE_PARAMETERS_END();
 
 	php_sock = Z_SOCKET_P(arg1);
 	ENSURE_SOCKET_VALID(php_sock);
@@ -1389,21 +1431,25 @@ PHP_FUNCTION(socket_recvfrom)
 	php_socket			*php_sock;
 	struct sockaddr_un	s_un;
 	struct sockaddr_in	sin;
-#if HAVE_IPV6
+#ifdef HAVE_IPV6
 	struct sockaddr_in6	sin6;
 #endif
-#ifdef HAVE_INET_NTOP
 	char				addrbuf[INET6_ADDRSTRLEN];
-#endif
 	socklen_t			slen;
 	int					retval;
 	zend_long				arg3, arg4;
 	const char			*address;
 	zend_string			*recv_buf;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "Ozllz|z", &arg1, socket_ce, &arg2, &arg3, &arg4, &arg5, &arg6) == FAILURE) {
-		RETURN_THROWS();
-	}
+	ZEND_PARSE_PARAMETERS_START(5, 6)
+		Z_PARAM_OBJECT_OF_CLASS(arg1, socket_ce)
+		Z_PARAM_ZVAL(arg2)
+		Z_PARAM_LONG(arg3)
+		Z_PARAM_LONG(arg4)
+		Z_PARAM_ZVAL(arg5)
+		Z_PARAM_OPTIONAL
+		Z_PARAM_ZVAL(arg6)
+	ZEND_PARSE_PARAMETERS_END();
 
 	php_sock = Z_SOCKET_P(arg1);
 	ENSURE_SOCKET_VALID(php_sock);
@@ -1457,17 +1503,13 @@ PHP_FUNCTION(socket_recvfrom)
 			ZSTR_LEN(recv_buf) = retval;
 			ZSTR_VAL(recv_buf)[ZSTR_LEN(recv_buf)] = '\0';
 
-#ifdef HAVE_INET_NTOP
 			address = inet_ntop(AF_INET, &sin.sin_addr, addrbuf, sizeof(addrbuf));
-#else
-			address = inet_ntoa(sin.sin_addr);
-#endif
 
 			ZEND_TRY_ASSIGN_REF_NEW_STR(arg2, recv_buf);
 			ZEND_TRY_ASSIGN_REF_STRING(arg5, address ? address : "0.0.0.0");
 			ZEND_TRY_ASSIGN_REF_LONG(arg6, ntohs(sin.sin_port));
 			break;
-#if HAVE_IPV6
+#ifdef HAVE_IPV6
 		case AF_INET6:
 			slen = sizeof(sin6);
 			memset(&sin6, 0, slen);
@@ -1512,7 +1554,7 @@ PHP_FUNCTION(socket_sendto)
 	php_socket			*php_sock;
 	struct sockaddr_un	s_un;
 	struct sockaddr_in	sin;
-#if HAVE_IPV6
+#ifdef HAVE_IPV6
 	struct sockaddr_in6	sin6;
 #endif
 	int					retval;
@@ -1521,9 +1563,15 @@ PHP_FUNCTION(socket_sendto)
 	bool           port_is_null = 1;
 	char				*buf, *addr;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "Oslls|l!", &arg1, socket_ce, &buf, &buf_len, &len, &flags, &addr, &addr_len, &port, &port_is_null) == FAILURE) {
-		RETURN_THROWS();
-	}
+	ZEND_PARSE_PARAMETERS_START(5, 6)
+		Z_PARAM_OBJECT_OF_CLASS(arg1, socket_ce)
+		Z_PARAM_STRING(buf, buf_len)
+		Z_PARAM_LONG(len)
+		Z_PARAM_LONG(flags)
+		Z_PARAM_STRING(addr, addr_len)
+		Z_PARAM_OPTIONAL
+		Z_PARAM_LONG_OR_NULL(port, port_is_null)
+	ZEND_PARSE_PARAMETERS_END();
 
 	php_sock = Z_SOCKET_P(arg1);
 	ENSURE_SOCKET_VALID(php_sock);
@@ -1558,7 +1606,7 @@ PHP_FUNCTION(socket_sendto)
 
 			retval = sendto(php_sock->bsd_socket, buf, ((size_t)len > buf_len) ? buf_len : (size_t)len, flags, (struct sockaddr *) &sin, sizeof(sin));
 			break;
-#if HAVE_IPV6
+#ifdef HAVE_IPV6
 		case AF_INET6:
 			if (port_is_null) {
 				zend_argument_value_error(6, "cannot be null when the socket type is AF_INET6");
@@ -1604,9 +1652,11 @@ PHP_FUNCTION(socket_get_option)
 	int				other_val;
 	zend_long			level, optname;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "Oll", &arg1, socket_ce, &level, &optname) == FAILURE) {
-		RETURN_THROWS();
-	}
+	ZEND_PARSE_PARAMETERS_START(3, 3)
+		Z_PARAM_OBJECT_OF_CLASS(arg1, socket_ce)
+		Z_PARAM_LONG(level)
+		Z_PARAM_LONG(optname)
+	ZEND_PARSE_PARAMETERS_END();
 
 	php_sock = Z_SOCKET_P(arg1);
 	ENSURE_SOCKET_VALID(php_sock);
@@ -1629,7 +1679,7 @@ PHP_FUNCTION(socket_get_option)
 		}
 		}
 	}
-#if HAVE_IPV6
+#ifdef HAVE_IPV6
 	else if (level == IPPROTO_IPV6) {
 		int ret = php_do_getsockopt_ipv6_rfc3542(php_sock, level, optname, return_value);
 		if (ret == SUCCESS) {
@@ -1662,6 +1712,9 @@ PHP_FUNCTION(socket_get_option)
 
 	if (level == SOL_SOCKET) {
 		switch (optname) {
+#ifdef SO_LINGER_SEC
+			case SO_LINGER_SEC:
+#endif
 			case SO_LINGER:
 				optlen = sizeof(linger_val);
 
@@ -1812,9 +1865,12 @@ PHP_FUNCTION(socket_set_option)
 	zval 					*l_onoff, *l_linger;
 	zval		 			*sec, *usec;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "Ollz", &arg1, socket_ce, &level, &optname, &arg4) == FAILURE) {
-		RETURN_THROWS();
-	}
+	ZEND_PARSE_PARAMETERS_START(4, 4)
+		Z_PARAM_OBJECT_OF_CLASS(arg1, socket_ce)
+		Z_PARAM_LONG(level)
+		Z_PARAM_LONG(optname)
+		Z_PARAM_ZVAL(arg4)
+	ZEND_PARSE_PARAMETERS_END();
 
 	php_sock = Z_SOCKET_P(arg1);
 	ENSURE_SOCKET_VALID(php_sock);
@@ -1834,7 +1890,7 @@ PHP_FUNCTION(socket_set_option)
 		HANDLE_SUBCALL(res);
 	}
 
-#if HAVE_IPV6
+#ifdef HAVE_IPV6
 	else if (level == IPPROTO_IPV6) {
 		int res = php_do_setsockopt_ipv6_mcast(php_sock, level, optname, arg4);
 		if (res == 1) {
@@ -1867,6 +1923,9 @@ PHP_FUNCTION(socket_set_option)
 	}
 
 	switch (optname) {
+#ifdef SO_LINGER_SEC
+		case SO_LINGER_SEC:
+#endif
 		case SO_LINGER: {
 			const char l_onoff_key[] = "l_onoff";
 			const char l_linger_key[] = "l_linger";
@@ -2059,14 +2118,17 @@ PHP_FUNCTION(socket_create_pair)
 	zval		retval[2], *fds_array_zval;
 	php_socket	*php_sock[2];
 	PHP_SOCKET	fds_array[2];
-	zend_long		domain, type, protocol;
+	zend_long		domain, type, checktype, protocol;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "lllz", &domain, &type, &protocol, &fds_array_zval) == FAILURE) {
-		RETURN_THROWS();
-	}
+	ZEND_PARSE_PARAMETERS_START(4, 4)
+		Z_PARAM_LONG(domain)
+		Z_PARAM_LONG(type)
+		Z_PARAM_LONG(protocol)
+		Z_PARAM_ZVAL(fds_array_zval)
+	ZEND_PARSE_PARAMETERS_END();
 
 	if (domain != AF_INET
-#if HAVE_IPV6
+#ifdef HAVE_IPV6
 		&& domain != AF_INET6
 #endif
 		&& domain != AF_UNIX) {
@@ -2074,9 +2136,18 @@ PHP_FUNCTION(socket_create_pair)
 		RETURN_THROWS();
 	}
 
-	if (type > 10) {
+	checktype = type;
+#ifdef SOCK_NONBLOCK
+	checktype &= ~(SOCK_CLOEXEC | SOCK_NONBLOCK);
+#endif
+
+	if (checktype > 10) {
 		zend_argument_value_error(2, "must be one of SOCK_STREAM, SOCK_DGRAM, SOCK_SEQPACKET,"
-			" SOCK_RAW, or SOCK_RDM");
+			" SOCK_RAW, or SOCK_RDM"
+#ifdef SOCK_NONBLOCK
+			" optionally OR'ed with SOCK_CLOEXEC, SOCK_NONBLOCK"
+#endif
+		);
 		RETURN_THROWS();
 	}
 
@@ -2126,9 +2197,11 @@ PHP_FUNCTION(socket_shutdown)
 	zend_long		how_shutdown = 2;
 	php_socket	*php_sock;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "O|l", &arg1, socket_ce, &how_shutdown) == FAILURE) {
-		RETURN_THROWS();
-	}
+	ZEND_PARSE_PARAMETERS_START(1, 2)
+		Z_PARAM_OBJECT_OF_CLASS(arg1, socket_ce)
+		Z_PARAM_OPTIONAL
+		Z_PARAM_LONG(how_shutdown)
+	ZEND_PARSE_PARAMETERS_END();
 
 	php_sock = Z_SOCKET_P(arg1);
 	ENSURE_SOCKET_VALID(php_sock);
@@ -2150,9 +2223,9 @@ PHP_FUNCTION(socket_atmark)
 	php_socket	*php_sock;
 	int r;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "O", &arg1, socket_ce) == FAILURE) {
-		RETURN_THROWS();
-	}
+	ZEND_PARSE_PARAMETERS_START(1, 1)
+		Z_PARAM_OBJECT_OF_CLASS(arg1, socket_ce)
+	ZEND_PARSE_PARAMETERS_END();
 
 	php_sock = Z_SOCKET_P(arg1);
 	ENSURE_SOCKET_VALID(php_sock);
@@ -2175,9 +2248,10 @@ PHP_FUNCTION(socket_last_error)
 	zval		*arg1 = NULL;
 	php_socket	*php_sock;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "|O!", &arg1, socket_ce) == FAILURE) {
-		RETURN_THROWS();
-	}
+	ZEND_PARSE_PARAMETERS_START(0, 1)
+		Z_PARAM_OPTIONAL
+		Z_PARAM_OBJECT_OF_CLASS_OR_NULL(arg1, socket_ce)
+	ZEND_PARSE_PARAMETERS_END();
 
 	if (arg1) {
 		php_sock = Z_SOCKET_P(arg1);
@@ -2196,9 +2270,10 @@ PHP_FUNCTION(socket_clear_error)
 	zval		*arg1 = NULL;
 	php_socket	*php_sock;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "|O!", &arg1, socket_ce) == FAILURE) {
-		RETURN_THROWS();
-	}
+	ZEND_PARSE_PARAMETERS_START(0, 1)
+		Z_PARAM_OPTIONAL
+		Z_PARAM_OBJECT_OF_CLASS_OR_NULL(arg1, socket_ce)
+	ZEND_PARSE_PARAMETERS_END();
 
 	if (arg1) {
 		php_sock = Z_SOCKET_P(arg1);
@@ -2262,9 +2337,9 @@ PHP_FUNCTION(socket_import_stream)
 	php_socket			 *retsock = NULL;
 	PHP_SOCKET			 socket; /* fd */
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "r", &zstream) == FAILURE) {
-		RETURN_THROWS();
-	}
+	ZEND_PARSE_PARAMETERS_START(1, 1)
+		Z_PARAM_RESOURCE(zstream)
+	ZEND_PARSE_PARAMETERS_END();
 	php_stream_from_zval(stream, zstream);
 
 	if (php_stream_cast(stream, PHP_STREAM_AS_SOCKETD, (void**)&socket, 1)) {
@@ -2309,9 +2384,9 @@ PHP_FUNCTION(socket_export_stream)
 	const char *protocol = NULL;
 	size_t protocollen = 0;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "O", &zsocket, socket_ce) == FAILURE) {
-		RETURN_THROWS();
-	}
+	ZEND_PARSE_PARAMETERS_START(1, 1)
+		Z_PARAM_OBJECT_OF_CLASS(zsocket, socket_ce)
+	ZEND_PARSE_PARAMETERS_END();
 
 	socket = Z_SOCKET_P(zsocket);
 	ENSURE_SOCKET_VALID(socket);
@@ -2325,7 +2400,7 @@ PHP_FUNCTION(socket_export_stream)
 	/* Determine if socket is using a protocol with one of the default registered
 	 * socket stream wrappers */
 	if (socket->type == PF_INET
-#if HAVE_IPV6
+#ifdef HAVE_IPV6
 		 || socket->type == PF_INET6
 #endif
 	) {
@@ -2406,9 +2481,12 @@ PHP_FUNCTION(socket_addrinfo_lookup)
 	struct addrinfo hints, *result, *rp;
 	php_addrinfo *res;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "S|s!a", &hostname, &service, &service_len, &zhints) == FAILURE) {
-		RETURN_THROWS();
-	}
+	ZEND_PARSE_PARAMETERS_START(1, 3)
+		Z_PARAM_STR(hostname)
+		Z_PARAM_OPTIONAL
+		Z_PARAM_STRING_OR_NULL(service, service_len)
+		Z_PARAM_ARRAY(zhints)
+	ZEND_PARSE_PARAMETERS_END();
 
 	memset(&hints, 0, sizeof(hints));
 
@@ -2470,9 +2548,9 @@ PHP_FUNCTION(socket_addrinfo_bind)
 	php_addrinfo	*ai;
 	php_socket		*php_sock;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "O", &arg1, address_info_ce) == FAILURE) {
-		RETURN_THROWS();
-	}
+	ZEND_PARSE_PARAMETERS_START(1, 1)
+		Z_PARAM_OBJECT_OF_CLASS(arg1, address_info_ce)
+	ZEND_PARSE_PARAMETERS_END();
 
 	ai = Z_ADDRESS_INFO_P(arg1);
 
@@ -2502,7 +2580,7 @@ PHP_FUNCTION(socket_addrinfo_bind)
 			}
 
 		case AF_INET:
-#if HAVE_IPV6
+#ifdef HAVE_IPV6
 		case AF_INET6:
 #endif
 			{
@@ -2533,9 +2611,9 @@ PHP_FUNCTION(socket_addrinfo_connect)
 	php_addrinfo	*ai;
 	php_socket		*php_sock;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "O", &arg1, address_info_ce) == FAILURE) {
-		RETURN_THROWS();
-	}
+	ZEND_PARSE_PARAMETERS_START(1, 1)
+		Z_PARAM_OBJECT_OF_CLASS(arg1, address_info_ce)
+	ZEND_PARSE_PARAMETERS_END();
 
 	ai = Z_ADDRESS_INFO_P(arg1);
 
@@ -2565,7 +2643,7 @@ PHP_FUNCTION(socket_addrinfo_connect)
 			}
 
 		case AF_INET:
-#if HAVE_IPV6
+#ifdef HAVE_IPV6
 		case AF_INET6:
 #endif
 			{
@@ -2594,9 +2672,9 @@ PHP_FUNCTION(socket_addrinfo_explain)
 	zval			*arg1, sockaddr;
 	php_addrinfo	*ai;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "O", &arg1, address_info_ce) == FAILURE) {
-		RETURN_THROWS();
-	}
+	ZEND_PARSE_PARAMETERS_START(1, 1)
+		Z_PARAM_OBJECT_OF_CLASS(arg1, address_info_ce)
+	ZEND_PARSE_PARAMETERS_END();
 
 	ai = Z_ADDRESS_INFO_P(arg1);
 
@@ -2622,7 +2700,7 @@ PHP_FUNCTION(socket_addrinfo_explain)
 				add_assoc_string(&sockaddr, "sin_addr", addr);
 				break;
 			}
-#if HAVE_IPV6
+#ifdef HAVE_IPV6
 		case AF_INET6:
 			{
 				struct sockaddr_in6 *sa = (struct sockaddr_in6 *) ai->addrinfo.ai_addr;
@@ -2652,9 +2730,10 @@ PHP_FUNCTION(socket_wsaprotocol_info_export)
 	zend_string *seg_name;
 	HANDLE map;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "Ol", &zsocket, socket_ce, &target_pid) == FAILURE) {
-		RETURN_THROWS();
-	}
+	ZEND_PARSE_PARAMETERS_START(2, 2)
+		Z_PARAM_OBJECT_OF_CLASS(zsocket, socket_ce)
+		Z_PARAM_LONG(target_pid)
+	ZEND_PARSE_PARAMETERS_END();
 
 	socket = Z_SOCKET_P(zsocket);
 	ENSURE_SOCKET_VALID(socket);
@@ -2707,9 +2786,9 @@ PHP_FUNCTION(socket_wsaprotocol_info_import)
 	php_socket	*php_sock;
 	HANDLE map;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &id, &id_len) == FAILURE) {
-		RETURN_THROWS();
-	}
+	ZEND_PARSE_PARAMETERS_START(1, 1)
+		Z_PARAM_STRING(id, id_len)
+	ZEND_PARSE_PARAMETERS_END();
 
 	map = OpenFileMapping(FILE_MAP_READ, FALSE, id);
 	if (map) {
@@ -2761,9 +2840,9 @@ PHP_FUNCTION(socket_wsaprotocol_info_release)
 	char *id;
 	size_t id_len;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &id, &id_len) == FAILURE) {
-		RETURN_THROWS();
-	}
+	ZEND_PARSE_PARAMETERS_START(1, 1)
+		Z_PARAM_STRING(id, id_len)
+	ZEND_PARSE_PARAMETERS_END();
 
 	RETURN_BOOL(SUCCESS == zend_hash_str_del(&(SOCKETS_G(wsa_info)), id, id_len));
 }
